#!/usr/bin/env python

from epics import caget, caput
from time import sleep, gmtime, strftime, localtime

#Define caput with wait function
def caputw(PVname,PVinput):
    return caput(PVname,PVinput,wait=True, timeout = 100000)

def dateandtime():
    return strftime("%a %d %b %Y %H:%M:%S",localtime())

def vortex(energy,time=1):
    cen, wid = vortex_Ecal(energy)
    set_roi1(cen,wid)
     
def set_roi1(cen,wid):
    caput('29idVORTEX:mca1.R0LO',round(cen-wid/2))
    caput('29idVORTEX:mca1.R0HI',round(cen+wid/2))
    
def set_roi2(cen,wid):
    caput('29idVORTEX:mca1.R1LO',round(cen-wid/2))
    caput('29idVORTEX:mca1.R1HI',round(cen+wid/2))
    
def vortex_Ecal(energy):
    cen = 12.8 + 0.72*energy
    wid = 120
    return cen, wid

def mcacounttime(time):
    caput('29idVORTEX:mca1.PRTM',time)
    
def runmca(time):
    caput('29idVORTEX:mca1.PRTM',time)
    sleep(0.1)
    caputw('29idVORTEX:mca1EraseStart',1)
    
def mcaoff():
    caput('29idd:Unidig1Bo1',1)
    
def mcaon():
    caput('29idd:Unidig1Bo1',0)

def savemca():
    caput('29idKappa:scan1.T2PV','29idVORTEX:scanH.EXSC')
    caput('29idKappa:scan1.D16PV','29idVORTEX:mca1.R0')
    caput('29idKappa:scan1.D17PV','29idVORTEX:mca1.R1')
    caput('29idKappa:scan1.D49PV','29idd:userTran1.D')
    caput('29idKappa:scan1.D50PV','29idd:userTran1.E')
    sleep(1)
    
def nosavemca():
    caput('29idKappa:scan1.T2PV','29idVORTEX:mca1EraseStart')
    caput('29idKappa:scan1.D16PV','29idVORTEX:mca1.R0')
    caput('29idKappa:scan1.D17PV','29idVORTEX:mca1.R1')
    caput('29idKappa:scan1.D49PV','29idd:userTran1.D')
    caput('29idKappa:scan1.D50PV','29idd:userTran1.E')
    sleep(1)
    
def nomca():
    caput('29idKappa:scan1.T2PV','')
    caput('29idKappa:scan1.D16PV','')
    caput('29idKappa:scan1.D17PV','')
    caput('29idKappa:scan1.D49PV','')
    caput('29idKappa:scan1.D50PV','')
    sleep(1)
    
    
def scan_num():
    scan = caget('29idKappa:saveData_message')
    loc = scan.find('.')
    if (loc==-1):
        scannum=0
    else:
        scannum =int(scan[loc-4:loc])
    return scannum
    
def mcafileinit():
   # mainpath = '/net/s4data/export/sector4/4idc/mda'
     scanrecpath = caget('29idKappa:saveData_subDir')
     scan = scan_num()+1
     caput('4idcVORTEX:saveData_subDir', '/'+ scanrecpath+'/S'+str(scan))
     caput('4idcVORTEX:saveData_scanNumber',1)

    
def chkmcasave():
    if (caget('29idKappa:scan1.T2PV')=='29idVORTEX:scanH.EXSC'):
        print('Saving mca files')
        mcafileinit()

    
