from epics import caget,caput

from ..instruments.electron_analyzer import EA
from ..instruments.shutters import main_shutter_close,branch_shutter_close
from ..instruments.diagnostics import diagnostics_all_out

##############################################################################################################
############################################          Shut down      #################################################
##############################################################################################################

def BL_Shutdown():
    BL_CloseAllShutters()
    BL_Valve_All(state="CLOSE")
    diagnostics_all_out()
    try:
        EA.off()
    except:
        print('EA is not running, visually confirm HV is off')
        
    caput("29iddau1:dau1:011:DAC_Set",0)    #RSXS HV = "OFF"

def BL_CloseAllShutters():
    main_shutter_close()
    branch_shutter_close('c')
    branch_shutter_close('d')

def BL_Valve_All(state="CLOSE"):
    ValveList=["V1A","V2A","V3B","V4B","V5B","V6B","V7C","V8C","V9C","V10C","V7D","V8D","V9D","V10D"]
    for Vname in ValveList:
        pv=BL_Valve2pv(Vname)+state+".VAL"
        caput(pv,1)

def BL_Valve(Vname,state="CLOSE"):
    pv=BL_Valve2pv(Vname)+state+".VAL"
    caput(pv,1)

def BL_Valve2pv(Vname):
    Valve={
        "V1A":"GV01","V2A":"GV02",                #A-Hutch
        "V3B":"GV03","V4B":"GV04","V5B":"GV05","V6B":"GV15",    #B-Branch
        "V7C":"GV06","V8C":"GV08","V9C":"GV09","V10C":"GV10",    #C-Branch
        "C-Turbo":"GV16",                    #ARPES
        "V7D":"GV11","V8D":"GV12","V9D":"GV13","V10D":"GV14",    #D-Branch
        "D-Turbo":"GV17","D-TES":"GV18"                #RSXS
    }
    pv="29id:BLEPS:"+Valve[Vname]+":"
    return pv