from math import floor
import time

from epics import caput
"""
Endstation class is used to contain information about ioc, branch, mode

it makes the default stuff work

this will prompt for the endstation and will set the default parameters, look at the 
init kwargs to see which defaults you can change.
"""
global BL
BL = None

class Endstation:
    """
    used for defining which endstation in which you are running
    for short names and ioc info

    BL = endstation('ARPES')
        BL.ioc = "ARPES" / "Kappa" / "Test"
        BL.mode = "User" / "Staff" / "No Xray"
        BL.branch = 'c' / 'd'
        BL.mda => mda scanRecord

    """

    def __init__(self,endstation_name,scan_ioc,xrays,BL_mode,mda_scanRecord,log_header,log_entries,Motors):
        """
        intializes the several beamline variables 

        endstation_name = 'ARPES' / 'Kappa'
        
        BL = Endstation()
            BL.endstation => endstation_name
            BL.xrays => True/False 
            BL.mode => previously: BL_Mode_Set
            BL.folder => 'b','c','d'
            BL.prefix => 'ARPES_','Kappa_'
            BL.ioc => previously: BL_ioc()
            BL.mda_filepath
            BL.log_header => dictionary with header list
            BL.log_entries => method to get pvs to be written to log file
            BL.Motors => motor calls
            
        """
        global BL
        endstations_list = ['ARPES','Kappa']   
        BL_mode_list = ['user','staff']

        if endstation_name in endstations_list:
            self.endstation=endstation_name
        else:
            print('Not a valid Endstation choice')
            print('Endstations: '+endstations_list)
            return

        if BL_mode in BL_mode_list:
            self.mode = BL_mode
            if BL_mode.lower == 'user':
                if endstation_name == 'ARPES':
                    self.folder = 'c'
                    self.prefix = 'ARPES_'
                elif endstation_name == 'Kappa':
                    self.folder = 'd'
                    self.prefix = 'Kappa_'
                #elif endstation_name == 'Octupole':
                    #self.folder = 'e'
            elif BL_mode.lower == 'staff':
                self.folder = 'b' #overwrite folder for staff mode
        else:
            print('Not a valid BL_mode choice')
            print('BL_modes: '+BL_mode_list) 
            return  

        self.xrays = xrays
        self.ioc = scan_ioc
        self.mda = mda_scanRecord
        self.log_header = log_header
        self.log_entries = log_entries
        self.Motors = Motors


    def set_logfile_path():
        """
        sets the default logfile path
        """

##############################################################################################################
##############################               BL commands            ##############################
##############################################################################################################
def BL_ioc():
    """
    returns the branch from the Endstation instance
    """
    return BL.ioc

def BL_mode():
    """
    returns the beamline mode, User / Staff / No_Xray
    """
    return BL.ioc

def BL_mda_prefix():
    """
    returns the mda file prefix
    """
    return BL.prefix

def BL_mda_filepath():
    """
    returns the mda file prefix
    """
    return BL.filepath
