import numpy as np
from time import sleep
from math import floor

from epics import caget, caput,PV

from iexcode.instruments.IEX_endstations import BL
from iexcode.instruments.staff import staff_detector_dictionary

from iexcode.instruments.files_and_folders import check_run,make_user_folders,folder_mda
from iexcode.instruments.staff import staff_detector_dictionary
from iexcode.instruments.logfile import logfile_name_set,logfile_header

from iexcode.instruments.conversions_constants import *
from iexcode.instruments.utilities import *

from iexcode.instruments.userCalcs import userStringSeq_clear, userStringSeq_pvs, userCalcOut_clear
from iexcode.instruments.scanRecord import ScanRecord
#from iexcode.instruments.xrays import xrays_reset,xrays_get_all

from iexcode.instruments.Motors import Motors
from iexcode.instruments.current_amplifiers import SRS, ca_reset_all, Keithley_pv
from iexcode.instruments.gate_valves import valve_close, branch_valves
from iexcode.instruments.shutters import branch_shutter_close
from iexcode.instruments.slits import slit3D_get

from iexcode.instruments.Kappa_det import Kappa_Detector
from iexcode.instruments.scalers import scaler_cts
from iexcode.instruments.MPA import MPA
from iexcode.instruments.spec_stuff import folders_spec


global Kappa_Motors
global tey, d3, d4, mesh, Kappa_scaler_pv, mpa
global tthdet

#############################################################################
def Kappa_init(set_folders=False,reset=False,**kwargs):
    """

    set_folders: sets the mda and EA folders; default => False
    reset: resets the scanRecord (detectors,triggers...)
    **kwargs:
        xrays: sets global variable; default => True
        BL_mode: 'user' / 'staff' => used for saving, detectors... 
    """
    kwargs.setdefault('scan_ioc','29idKappa:')
    kwargs.setdefault('xrays',True)
    kwargs.setdefault('BL_mode','user')

    #scan
    if kwargs['BL_mode']=='staff':
        detector_dictionary = staff_detector_dictionary()
    else:
        detector_dictionary = Kappa_detector_dictionary()

    mda_scanRecord = ScanRecord(kwargs['scan_ioc'],detector_dictionary,
    Kappa_trigger_dictionary(),Kappa_scan_before_sequence(),Kappa_scan_after_sequence())

    #endstation
    BL=Endstation('Kappa',kwargs['scan_ioc'],kwargs['xrays'],kwargs['BL_mode'],mda_scanRecord,Kappa_log_header,Kappa_log_entries)

    #global detectors
    tey = SRS("29idMZ0:scaler1.S2", '29idd:A1')
    d3 = SRS("29idMZ0:scaler1.S3", '29idd:A2')
    d4 = SRS("29idMZ0:scaler1.S4", '29idd:A3')
    mesh = SRS("29idMZ0:scaler1.S14", '29idd:A4')
    Kappa_scaler_pv = '29idMZ0:scaler1.CNT'
    tth_pv = Kappa_motor_dictionary('tth')[3]
    mpa = MPA(tth_pv)

    tthdet = Kappa_Detector(Kappa_Motors)

    #setting folders
    if 'set_folders':
        if BL.mode == 'staff':
            user_name = 'staff'
        else:
            user_name = input('user name: ')
        folders_Kappa(user_name,**kwargs)
    
    #resetting
    if 'reset':
        Kappa_reset()

    #motors
    physical_motors = ['x','y','z','tth','kth','kap','kphi']
    psuedo_motors = ['th','chi','phi']
    Kappa_Motors = Motors('Kappa',Kappa_motor_dictionary(),physical_motors,psuedo_motors)

##############################################################################################################
##############################                    detectors and motors                ##############################
##############################################################################################################
def Kappa_detector_list():
    """
    list of detectors to trigger 

    Previously: part of Detector_List
    """
    ca_list=[]
    return ca_list


def Kappa_detector_dictionary(**kwargs):
    """
    returns a dictionary of the default detectors

    **kwargs
    add_vortex: to specifiy to add the vortex detectors to the dictionary (True/False)

    Previously: Detector_Default
    """
    kwargs.setdefault('add_vortex',False)
    det_dict={}
    vortex={
        15:"",
        16:"",
        17:"",
    }
    sample_temp={
        23:"29idd:LS331:TC1:SampleA",
        24:"29idd:LS331:TC1:SampleB",
    }
    m3r={
        25:"29id_ps6:Stats1:CentroidX_RBV",
        26:"29id_ps6:Stats1:SigmaX_RBV",
        27:"29id_ps6:Stats1:CentroidTotal_RBV",
    }
    scalers={
        31:"29idMZ0:scaler1.S14",
        32:"29idMZ0:scaler1.S2",
        33:"29idMZ0:scaler1.S3",
        34:"29idMZ0:scaler1.S4",
        35:"29idMZ0:scaler1.S5",
        36:"29idMZ0:scaler1_calc1.B",
        37:"29idMZ0:scaler1_calc1.C",
        38:"29idMZ0:scaler1_calc1.D",
        39:"29idMZ0:scaler1_calc1.E",
    }
    mpa_dets={
        30:"29iddMPA:det1:TotalRate_RBV",
        41:"29iddMPA:Stats1:Total_RBV",
        42:"29iddMPA:Stats2:Total_RBV",
        43:"29iddMPA:Stats3:Total_RBV",
        44:"29iddMPA:Stats4:Total_RBV",
        45:"29iddMPA:Stats5:Total_RBV",
    }
    hkl={
        46:'<H>',
        47:'<K>',
        48:'<L>'
    } 
    motors={
        51:"29idKappa:m8.RBV",
        52:"29idKappa:m7.RBV",
        53:"29idKappa:m1.RBV",
        54:"29idKappa:m9.RBV",
        55:"29idKappa:Euler_ThetaRBV",
        56:"29idKappa:Euler_ChiRBV",
        57:"29idKappa:Euler_PhiRBV",
    }
    #hkl are listed just a place holders, they are filled in by thier scanning functions
    det_dict.update(sample_temp)
    det_dict.update(scalers)
    det_dict.update(mpa_dets)
    det_dict.update(motors)
    if kwargs['add_vortex']:
        det_dict.update(vortex)

    return det_dict


def Kappa_motor_dictionary(name):
    """
    motor_dictionary = {name:[rbv,val,spmg,pv]} for physical and psuedo/Euler motors
    usage:
        KappaS_PVmotor('x') => ['29idKappa:m2.RBV', '29idKappa:m2.VAL', '29idKapp:m2.SPMG','29idKappa:m2']
    """
    motor_nums={
        'x':2,
        'y':3,
        'z':4,
        'tth':9,
        'kth':8,
        'kap':7,
        'kphi':1,
      }
    Euler_motors={
        'th':'29idKappa:Euler_Theta',
        'chi':'29idKappa:Euler_Chi',
        'phi':'29idKappa:Euler_Phi'
    }
    motor_dictionary = {}
    for name in motor_nums.keys():
        pv = '29idKappa:m'+str(motor_nums[name])
        motor_dictionary.update({name:[pv+'.VAL',pv+'.SPMG',pv]})    
    
    for name in Euler_motors.keys():
        pv = Euler_motors[name]
        motor_dictionary.update({name:[pv+'RBV',pv,'',pv]})
    
    return motor_dictionary

def Kappa_extra_pvs():
    """
    used to get the PV associated with a given pnuemonic

    """
    d={
        "TA":"29idd:LS331:TC1:Control",
        "TB":"29idd:LS331:TC1:SampleB",
        "det_nam":'29idKappa:userStringSeq6.STR1',
        "HV":'29idKappa:userCalcOut10.OVAL',
        "centroid":'29id_ps6:Stats1:CentroidX_RBV'        
    }
    return d

def Kappa_kth_offset_get():
    caget("29idKappa:userCalcOut1.G")

def Kappa_kth_offset_set(val):
    caput("29idKappa:userCalcOut1.G",val)


#############################################################################################################
##############################                 setting folder                   ##############################
##############################################################################################################
def folders_Kappa(user_name,**kwargs):
    """
    Create and sets (if create_only=False) all the folders the current run and ARPES user 
    Sets the FileName for logging to be "UserName/YYYYMMDD_log.txt using logname_Set()"
    
    **kwargs:
        set_folders = True (default); set the mda and EA scanRecords
                    = False; only makes the folders (was create_only)
        run: run cycle e.g. 2022_1 (default => check_run() )
        ftp = True / False (default); print what needs to be added to the ftps crontab and create additional ftp folders
        mda_ioc: will overwrite the default ioc for mda scans
        debug
    """
    kwargs.setdefault('set_folders',False)
    kwargs.setdefault('run',check_run())
    kwargs.setdefault('ftp',False)
    kwargs.setdefault('debug',False)

    run = kwargs['run']

    if kwargs['debug']:
        print("run,folder,user_name,ioc,ftp: ",run,BL.folder,user_name,BL.ioc,kwargs['ftp'])
    
    # Create User Folder:
    make_user_folders(run,BL.folder,user_name,BL.endstation,ftp=kwargs['ftp'])
    sleep(5)

    if kwargs["set_folders"]:
        # Set up MDA folder:
        folder_mda(run,BL.folder,user_name,BL.prefix,BL.ioc)
        logfile_name_set(BL.endstation)
        logfile_header(BL.endstation,BL.ioc,Kappa_log_header())


        # Set up SPEC folder:
        folders_spec(run,BL.folder,user_name)

        # Set up MPA folder:
        #Folder_MPA(run,BL.folder,user_name)

    #resetting
    if 'reset':
        Kappa_reset()

def Kappa_reset():
    """
    resets scanRecord, current amplifiers, mono limits and lakeshore
    """
    #resetting the scanRecord
    BL.mda.reset()

    #resetting the current amplifiers
    if BL.xray:
        ca_reset_all()
    
    #resetting mono and anyother beamline stuff
    if BL.xrays:
        xrays_reset()

    #reseting the Kappa Lakeshore

    #motors home and sync
    SmarAct_motors_home()
    PI_motors_sync()
    Euler_motors_sync()

def Kappa_reminder_list(ioc):
    """
    resets scanRecord, current amplifiers, mono limits  and synchs motors
    """
    BL.mda.reset()
    if BL.xray:
        ca_reset_all()





##############################################################################################################
##############################                    get all                 ##############################
##############################################################################################################
def Kappa_get_all(verbose=True):
    """
    returns a dictionary with the current status of the Kappa endstation and exit slit
    """
    vals = {}

    #sample postion
    motor_dictionary = Kappa_motor_dictionary()
    for motor in motor_dictionary.keys():
        vals[motor]=Kappa_Motors.get(motor,verbose=False)

    #endstation pvs
    extra_pvs = Kappa_extra_pvs()
    for key in extra_pvs.keys():
        vals.update(key,caget(extra_pvs[key]))

    #beamline info
    if BL.xray:
        beamline_info = xrays_get_all()
        #vals.update(beamline_info) 

    mesh.get()
    vals.update({'mesh':mesh.current})

    if verbose:
        for key in vals:
            print(key+" = "+vals[key])
    return vals

##############################################################################################################
##############################                         logging                  ##############################
##############################################################################################################
def Kappa_log_header():
    """
    header for the log file
    """
    h = "scan,motor,start,stop,step,x,y,z,tth,kth,kap,kphi,TA,TB,"
    h += "hv,exit_slit,GRT,ID_SP,ID_RBV,ID_Mode,ID_QP,TEY,mesh,det_name,mpa_HV,m3r_centroid,time,comment"
    header_list = {'Kappa':h}
    return header_list 

def Kappa_log_entries():
    """
    enstation info for log file

    Previously: scanlog
    """
    vals = Kappa_get_all(verbose=False)
    x = vals['x']
    y = vals['y']
    z = vals['z']
    tth = vals['tth']
    kth = vals['kth']
    kap = vals['kap']
    kphi = vals['kphi']

    TA = vals['TA']
    TB = vals['TB']
    mesh.get()
    tey_current = tey.current
    mesh_current = mesh.current
    det_name = tthdet.name
    mpa_HV = mpa.HV_get()
    m3r_centroid = vals['kphi']

    entry_list = ["x","y","z","tth","kth","kap","kphi","TA","TB","TEY","mesh","det_name","mpa_HV","m3r_centroid"]
    pv_list = [ x, y, z, tth, kth, kap,kphi, TA, TB,tey_current,mesh_current,det_name,mpa_HV,m3r_centroid]  
    format_list = [".2f",".2f",".2f",".2f",".2f",".2f",".2f",".2f",".2f","1.2e","1.2e","s",".0f",".0f"]

    return entry_list,pv_list,format_list

##############################################################################################################
##############################             Kappa scanRecord           ##############################
##############################################################################################################
def Kappa_scan_before_sequence(**kwargs):
    """
    writes the user string sequence to happen at the beginning of a scan
    returns before_scan_pv = pv for userStringSeq for before scan

    **kwargs
        seq_num: userStringSeq number in ioc => 9 (default)

    Previously: BeforeScan_StrSeq
    """
    kwargs.setdefault(seq_num,9)
    seq_num=kwargs['seq_num']

    before_scan_pv,before_scan_proc = userStringSeq_pvs(BL.ioc, seq_num)

    #clear and write the before scan user sequence
    userStringSeq_clear(BL.ioc,seq_num)
    caput(before_scan_pv+".DESC","Before Scan")

    #This is where you'd do something if need (CA -> 'Passive', etc)
    pv=''
    cmd=''
    caput(before_scan_pv+".LNK" +str(1),pv)
    caput(before_scan_pv+".STR" +str(1),cmd)

    return before_scan_proc

def Kappa_scan_after_sequence(scan_dim, **kwargs):
    """
    writes the user string sequence to happen at the end of a scan
    returns after_scan_pv = pv for userStringSeq for after scan

    **kwargs
        seq_num: userStringSeq number in ioc => 10 (default)
        snake: for snake scanning => False (default)

    Previously: AfterScan_StrSeq
    """
    kwargs.setdefault(seq_num,10)
    kwargs.setdefault('snake',False)
    seq_num=kwargs['seq_num']

    after_scan_pv,after_scan_proc = userStringSeq_pvs(BL.ioc, seq_num)
    
    #clear and write the after scan user sequence
    userStringSeq_clear(BL.ioc,seq_num)
    caput(after_scan_pv+".DESC","After Scan")
        
    scan_pv = BL.ioc+"scan"+str(scan_dim)
    ## Put scan record back in absolute mode
    caput(after_scan_pv+".LNK2",scan_pv+".P1AR")
    caput(after_scan_pv+".STR2","ABSOLUTE")

    ## Put Positionner Settling time to 0.1s
    caput(after_scan_pv+".LNK3",scan_pv+".PDLY NPP NMS")
    caput(after_scan_pv+".DO3",0.1)

    ## Clear DetTriggers 2 to 4:
    caput(after_scan_pv+".LNK4",scan_pv+".T2PV NPP NMS")    #remove trigger 2 (used for EA) after scan

    if kwargs['snake']:
        snake_dim = scan_dim - 1
        #snake_proc= Kappa_snake_pv(snake_dim,enable=True)
        #caput(after_scan_pv+".LNK10",Kappa_snake_pv()+"PP NMS")
        #caput(after_scan_pv+".D10",1)

    return after_scan_proc

def Kappa_detector_triggers_sequence(**kwargs):    # do we need to add 29idb:ca5 ???
    """
    """
    kwargs.setdefault(seq_num,8)
    seq_num=kwargs['seq_num']

    detector_triggers_pv,detector_triggers_proc = userStringSeq_pvs(BL.ioc, seq_num)
       
    #clear the userStringSeq
    userStringSeq_clear(BL.ioc,seq_num=kwargs['seq_num'])
    caput(detector_triggers_pv+".DESC","Kappa_Trigger1")

    scaler_pv = Kappa_scaler_pv

    caput(detector_triggers_pv+".LNK" +str(1),scaler_pv)
    caput(detector_triggers_pv+".WAIT"+str(1),"After"+str(last))
    
    ca_list = Kappa_detector_list
    last = len(ca_list)
    for i,ca in enumerate(ca_list):
        ca_pv = Keithley_pv(ca[0], ca[1])+':read.PROC CA NMS'
        caput(detector_triggers_pv+".LNK" +str(i+2),ca_pv)
        caput(detector_triggers_pv+".WAIT"+str(i+2),"After"+str(last))

    return detector_triggers_proc

def Kappa_trigger_dictionary():
    """
    need to do something
    """
    trigger_dictionary = {
        1:Kappa_detector_triggers_sequence(),
    }
    return trigger_dictionary




##############################################################################################################
##############################        Kappa  Motor Scan Set Up        ##############################
##############################################################################################################


def Kappa_sample_Euler_list():
    """
    returns list of motor names used by mvsample
    """
    motor_list = ['th','chi','phi']
    return motor_list


def Kappa_4c_mprint():
    """
    returns the dictionary of the current sample position in 4c units 
    
    motors=['th','chi','phi']

    Previously: diffracto?
    """
    positions={}
    motors=Kappa_sample_Euler_list()
    for motor in motors:
        positions.update(motor,Kappa_motor_dictionary()(motor)[0])
    return positions

def Kappa_4c_move(th_chi_phi_list):
    """
    moves the sample in 4c (Euler) units
    Previously: mv4C
    """
    motor_list=Kappa_sample_Euler_list()
    for motor,i in enumerate(motor_list):
        Kappa_Motors.move(motor,motor_list[i],wait=True,verbose=False)
    
    for motor,i in enumerate(motor_list):
        Kappa_Motors.get(motor,verbose=True)

   

def SmarAct_motor_list():
    """
    returns a list of the motor names for the SmarAct motors in the Kappa
    """
    return ['x','y','z']

def SmarAct_motors_home():
    """
    Homes the piezo (x,y,z). Home position is middle of travel

    Previously: Home_SmarAct_Motor
    """
    motor_dictionary = Kappa_motor_dictionary()
    for motor in SmarAct_motor_list():
        pv = motor_dictionary[motor][3]
        caput(pv+'.HOMF',1)
        sleep(10)
    print('SamrAct motors VAL homed')

def SmarAct_enable():
    motor_dictionary = Kappa_motor_dictionary()
    for motor in SmarAct_motor_list():
        spmg = motor_dictionary[motor][2]
        caput(spmg,3)  # 3=Go

def SmarAct_disable():
    motor_dictionary = Kappa_motor_dictionary()
    for motor in SmarAct_motor_list():
        spmg = motor_dictionary[motor][2]
        caput(spmg,0)  # 1=Stop

def PI_motor_list():
    """
    returns a list of the motor names for the SmarAct motors in the Kappa
    """
    return ['kap','kth','tth']

def PI_motors_sync():
    motor_dictionary = Kappa_motor_dictionary()
    for motor in PI_motor_list():
        val_pv = motor_dictionary[motor][1]
        rbv_pv = motor_dictionary[motor][0]
        current_rbv=caget(rbv_pv)
        caput(val_pv,current_rbv)
    print('PI motors VAL synced to RBV')

def Euler_motors_sync():
    """
    Syncs the Euler motores

    Previously: Sync_Euler_Motor
    """
    caput('29idKappa:Kappa_sync.PROC',1)
    sleep(1)
    caput('29idKappa:Kappa_sync.PROC',1)
    print('Euler motors VAL/RBV synced to physical motors')

def Kappa_kphi_reset_zero(val):
    """
    resets the zero for the kphi motor

    """
    kphi_pv = Kappa_motor_dictionary()['kphi'][3]
    caput(kphi_pv+".SET",1)    # 1 = Set
    sleep(0.5)
    caput("kphi_pv.VAL",val)
    sleep(0.5)
    caput(kphi_pv+".SET",0)    # 0 = Use
    print("\nkphi has been reset to " +str(val))

def Kappa_tth_reset_zero():
    """
    resets the zero for the tth motor
    """
    tthdet.tth0_set()



def Kappa_th2th_scan_sensitivity(th_table,gain_num_table,gain_unit_table,**kwargs):
    """
    Scans th2th with variable gain on SRS# (srs_num).
    Gain is specified for a given th2th range by building tables as follow:

    th_table = np.array([])
    gain_num_table = np.array([])
    gain_unit_table = np.array([])

    Example:
        for i in RangeUp(10,20,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(2))
            gu_table = np.append(gu_table, unit_dict('mA'))
        for i in RangeUp(20.5,40,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(1))
            gu_table = np.append(gu_table, unit_dict('uA'))
        for i in RangeUp(40.5,60,0.5):
            th_table = np.append(th_table, i)
            gn_table = np.append(gn_table, gain_dict(10))
            gu_table = np.append(gu_table, unit_dict('pA'))
    
    kwargs:
        detector (default tey)
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default)
        execute: True/False to start the scan => True (default)
       
    Previoulsy: Scan_th2th_sensitivity
    """
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)

    if 'detector' not in kwargs:
        detector = tey
    else:
        detector = kwargs['detector']

    kth_offset = Kappa_kth_offset_get()

    motor_dictionary = Kappa_motor_dictionary()
    kth_val,kth_rbv,kth_spmg,kth_pv = motor_dictionary['kth']
    tth_val,tth_rbv,tth_spmg,tth_pv = motor_dictionary['tth']

    detector_pv = detector._srs_pv
    gain_num_pv = detector_pv + "sens_num.VAL"
    gain_unit_pv= detector_pv + "sens_unit.VAL"

    kth_table = th_table + kth_offset
    tth_table = 2*th_table

    kwargs.update("positioner_num",1)
    BL.mda.fillin_table(kth_val,kth_rbv,kth_table,**kwargs)

    kwargs.update("positioner_num",2)
    BL.mda.fillin_table(tth_val,tth_rbv,tth_table,**kwargs)

    kwargs.update("positioner_num",3)
    BL.mda.fillin_table(gain_num_pv,"",gain_num_table,**kwargs)

    kwargs.update("positioner_num",4)
    BL.mda.fillin_table(gain_unit_pv,"",gain_unit_table,**kwargs)

    print("\nDon't forget to clear extra positionners at the end of the scan if you were to abort the script using the function:")
    print("                Clear_Scan_Positioners('Kappa',1)")
 
    scaler_cts(kwargs['cts'],verbose=False)
    if kwargs['execute']:
        BL.mda.go(**kwargs)

        #clean up after scan
        BL.mda.table_reset_after(**kwargs)
        scaler_cts(verbose=False)

def Kappa_scan_th2th(tth_start,tth_stop,tth_step,th_offset,**kwargs):
    """
    Used for a linear (not table) scan where th =  tth /2 + th_offset
    **kwargs
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default) 
        execute: True/False to start the scan => True (default)

    Previously: scanth2th
    """
    kwargs.setdefault('scan_dim',1)
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)
    
    th_start = (tth_start)/2+th_offset
    th_stop = (tth_stop)/2+th_offset
    th_step = int(tth_step/2)
    
    print('tth: '+str(tth_start)+"/"+str(tth_stop)+"/"+str(tth_step))
    print('th: '+str(th_start)+"/"+str(th_stop)+"/"+str(tth_step/2.0))
    
    #write to the scanRecord
    th_val,th_rbv,th_spmg,th_pv = Kappa_motor_dictionary['th']
    tth_val,tth_rbv,tth_spmg,tth_pv = Kappa_motor_dictionary['tth']

    kwargs.update("positioner_num",2)
    BL.mda.fillin(th_val,th_rbv,th_start,th_stop,th_step,**kwargs)
    
    kwargs.update("positioner_num",1)
    BL.mda.fillin(tth_val,tth_rbv,tth_start,tth_stop,tth_step,**kwargs)
    
    scaler_cts(kwargs['cts'],verbose=False)
    if kwargs['execute']:
        BL.mda.go(**kwargs)

        #clean up after scan
        scaler_cts(verbose=False)


def scan_th2th_table(tth_table,th0,**kwargs):
    """
    Create a table for tth, e.g.:
        mytable_tth=[]
        for q in range(2,41,1):
            mytable_tth.append(q)
        print(mytable_tth)
        
    th0 = th value at grazing incidence
    cts = counting time (default is 1s)
    
    **kwargs
        scan_dim: 1  (default)
        cts: integration time for scalers and mpa/mcp => 0.1 (default) 
        execute: True/False to start the scan => True (default)

    Previously: scanth2th_table

    """
    kwargs.setdefault('cts',0.1)
    kwargs.setdefault('execute',True)
  
    tth_table=np.asarray(tth_table)
    th_table=tth_table/2.0+th0

    th_val,th_rbv,th_spmg,th_pv = Kappa_motor_dictionary['th']
    tth_val,tth_rbv,tth_spmg,tth_pv = Kappa_motor_dictionary['tth']

    #write to the scanRecord
    kwargs.update("positioner_num",1)
    BL.mda.fillin_table(tth_val,tth_rbv,tth_table,**kwargs)

    kwargs.update("positioner_num",2)
    BL.mda.fillin_table(th_val,th_rbv,th_table,**kwargs)
    BL.mda.positioner_after_scan(after="STAY")


    scaler_cts(kwargs['cts'],verbose=False,**kwargs)
    if kwargs['execute']:
        BL.mda.go(**kwargs)

        #clean up after scan
        BL.mda.table_reset_after()
        scaler_cts(verbose=False)
        BL.mda.positioner_after_scan(after="PRIOR POS",**kwargs)




#############################################################################################################
##############################             Preset Positions        ##############################
##############################################################################################################
def KappaTransfer_StrSeq():
    #User= [    DESC,        x,        y,      z,        tth,       kth,       kap,    kphi]
    User = ["Kappa Transfer",0, -2650, -650, 0, 57, 0, -88]
    n=4
    KappaPreset_StrSeq(n,User)


def KappaGrazing_StrSeq(): #Need to determine positions and then add to the Kappa graphic
    #Dial= [    DESC,        x,        y,      z,        tth,       kth,       kap,    kphi]
    User = ["Kappa Grazing",0, 0, 0, 0, 57.045, 134.76,57.045]
    n=3
    KappaPreset_StrSeq(n,User)

def Kappa_ResetPreset():
    KappaGrazing_StrSeq()
    KappaTransfer_StrSeq()










##############################################################################################################
##############################             Kappa light       ##############################
############################################################################################################## 
def Kappa_light(ON_OFF):
    """
    Previously:light
    """
    light_pv = '29idd:Unidig1Bo0'
    if ON_OFF.lower() == 'on':
        light=0
    elif ON_OFF.lower() == 'off':
        light=1
    caput(light_pv,light)
    print(("Turning light "+ON_OFF+"."))




def Kappa_detector_triggers_strSeq(**kwargs):    # do we need to add 29idb:ca5 ???
    """
    """
    kwargs.setdefault(seq_num,8)
    seq_num=kwargs['seq_num']
    
    detector_triggers_pv,detector_triggers_proc = userStringSeq_pvs(BL.ioc, seq_num)
       
    #clear the userStringSeq
    userStringSeq_clear(BL.ioc,seq_num=kwargs['seq_num'])
    caput(detector_triggers_pv+".DESC","Kappa_Trigger1")

    #no triggers see ARPES_detector_triggers_sequence for example

    return detector_triggers_proc






def KappaPreset_StrSeq(n,User):
    scanIOC="Kappa"
    motorIOC="29idKappa:"
    motor = ["m2","m3","m4","m9","m8","m7","m1"]
    strSeq_pv = userStringSeq_clear(BL.mda,n)

    if User[0] == "Kappa Grazing":  phi0= 0
    if User[0] == "Kappa Transfer": phi0= 57
    caput(strSeq_pv+".DESC",User[0])
    caput(strSeq_pv+".LNK1", "29idKappa:userCalcOut9.A CA NMS")       # MPA HV pV
    caput(strSeq_pv+".DO1",0)                                       # MPA HV = 0
    caput(strSeq_pv+".WAIT1","Wait")                                # Wait for completion
    caput(strSeq_pv+".LNK2", "29idKappa:m1.VAL CA NMS")               # phi = phi0
    caput(strSeq_pv+".DO2",phi0)                                       
    caput(strSeq_pv+".WAIT2","Wait")                                # Wait for completion
    for i in range(3,10):
        caput(strSeq_pv+".LNK"+str(i),motorIOC+motor[i-3]+".VAL CA NMS")
        caput(strSeq_pv+".DO"+str(i),User[i-2])
        if i < 9:
            caput(strSeq_pv+".WAIT"+str(i),"After8")




    
 
    
    
def Bragg_Angle_CalcOut(d,eV,l):
    n=7
    userCalcOut_pv = userCalcOut_clear(BL.mda,n)

    h=4.135667516e-15
    c=299792458
    f=h*c*1e9*10
    caput(userCalcOut_pv+"DESC","Bragg_Angle")
    caput(userCalcOut_pv+"A",d)
    caput(userCalcOut_pv+"B",l)
    caput(userCalcOut_pv+"C",eV)
    caput(userCalcOut_pv+"D",np.pi)
    caput(userCalcOut_pv+"E",f)
    caput(userCalcOut_pv+".CALC$","ASIN(B*E/(C*"+str(2)+"*A))*"+str(180)+"/D")

##############################################################################################################
########################        Scan Temp and Pressure           ##############################
##############################################################################################################
def Kappa_temperature_pressure_scan(scan_dim=1):
    """
    starts scan to monitor temperature and pressure in the kappa chamber

    Previously: Kappa_ScanTempPres
    """

    pv="29id"+BL.mda.ioc+":scan"+str(scan_dim)
    #Clear all scan pvs
    caput(pv+".CMND",6)
    #Set detectors
    caput(pv+".D01PV","29idd:tc1:getVal_A.VAL")
    caput(pv+".D02PV","29idd:tc1:getVal_B.VAL")
    caput(pv+".D03PV","29idb:VS11D.VAL")
    #time scan
    BL.mda.time_go()



##############################################################################################################
##############################             Kappa safestate        ##############################
##############################################################################################################
def Kappa_safe_state(**kwargs):
    """
    puts the C-branch in a safe state, 
    **kwargs
        MPA_off = True; turns off EA HV
        shutter_close = True; closes the D-shutter
        valve_close = True; closes the D-valve
    """
    kwargs.setdefault("MPA_off",True)
    kwargs.setdefault("shutter_close",True)
    kwargs.setdefault("valve_close",True)
    
    if kwargs["mpa_off"]:
        try:
            mpa.HV_off()
        except:
            print('MPA is not running')

    if kwargs['valve_close']:
       valve_close(branch_valves('d'), verbose=True)

    if kwargs['shutter_close']:
        branch_shutter_close('d')
    
##############################################################################################################
##############################              Kappa motors       ##############################
##############################################################################################################
### x ###
def mvx(val,**kwargs):
    """ moves x motor in the endstation"""
    Kappa_Motors.move('x',val,**kwargs)

def dmvx(val):
    """ relative x motor in the endstation"""
    Kappa_Motors.move('x',val,relative=True)

def scanx(start,stop,step,**kwargs):
    """ scans the x motor in the endstation"""
    Kappa_Motors.scan("x",start,stop,step,**kwargs)

def dscanx(start,stop,step):
    """relative scans of the x motor in the endstation """
    Kappa_Motors.scan("x",start,stop,step,relative=True)

### y ###
def mvy(val,**kwargs):
    """ moves y motor in the endstation"""
    Kappa_Motors.move('y',val,**kwargs)

def dmvy(val):
    """ relative y motor in the endstation"""
    Kappa_Motors.move('y',val,relative=True)

def scany(start,stop,step,**kwargs):
    """ scans the y motor in the endstation"""
    Kappa_Motors.scan("y",start,stop,step,**kwargs)

def dscany(start,stop,step):
    """relative scans of the y motor in the endstation """
    Kappa_Motors.scan("y",start,stop,step,relative=True)

### z ###
def mvz(val,**kwargs):
    """ moves z motor in the endstation"""
    Kappa_Motors.move('z',val,**kwargs)

def dmvz(val):
    """ relative z motor in the endstation"""
    Kappa_Motors.move('z',val,relative=True)

def scanz(start,stop,step,**kwargs):
    """ scans the z motor in the endstation"""
    Kappa_Motors.scan("z",start,stop,step,**kwargs)

def dscanz(start,stop,step):
    """relative scans of the z motor in the endstation """
    Kappa_Motors.scan("z",start,stop,step,relative=True)

### kth ###
def mvkth(val,**kwargs):
    """ moves kth motor in the endstation"""
    Kappa_Motors.move('kth',val,**kwargs)

def dmvkth(val):
    """ relative kth motor in the endstation"""
    Kappa_Motors.move('kth',val,relative=True)

def scankth(start,stop,step,**kwargs):
    """ scans the kth motor in the endstation"""
    Kappa_Motors.scan("kth",start,stop,step,**kwargs)

def dscankth(start,stop,step):
    """relative scans of the kth motor in the endstation """
    Kappa_Motors.scan("kth",start,stop,step,relative=True)

### kphi ###
def mvkphi(val,**kwargs):
    """ moves kphi motor in the endstation"""
    Kappa_Motors.move('kphi',val,**kwargs)

def dmvkphi(val):
    """ relative kphi motor in the endstation"""
    Kappa_Motors.move('kphi',val,relative=True)

def scankphi(start,stop,step,**kwargs):
    """ scans the kphi motor in the endstation"""
    Kappa_Motors.scan("kphi",start,stop,step,**kwargs)

def dscankphi(start,stop,step):
    """relative scans of the kphi motor in the endstation """
    Kappa_Motors.scan("kphi",start,stop,step,relative=True)   

### kap ###
def mvkap(val,**kwargs):
    """ moves kap motor in the endstation"""
    Kappa_Motors.move('kap',val,**kwargs)

def dmvkap(val):
    """ relative kap motor in the endstation"""
    Kappa_Motors.move('kap',val,relative=True)

def scankap(start,stop,step,**kwargs):
    """ scans the kap motor in the endstation"""
    Kappa_Motors.scan("kap",start,stop,step,**kwargs)

def dscankap(start,stop,step):
    """relative scans of the kap motor in the endstation """
    Kappa_Motors.scan("kap",start,stop,step,relative=True)

### tth ###
def mvtth(val,**kwargs):
    """ moves tth motor in the endstation"""
    Kappa_Motors.move('tth',val,**kwargs)

def dmvtth(val):
    """ relative tth motor in the endstation"""
    Kappa_Motors.move('tth',val,relative=True)

def scantth(start,stop,step,**kwargs):
    """ scans the tth motor in the endstation"""
    Kappa_Motors.scan("tth",start,stop,step,**kwargs)

def dscantth(start,stop,step):
    """relative scans of the tth motor in the endstation """
    Kappa_Motors.scan("tth",start,stop,step,relative=True)

### th ###
def mvth(val,**kwargs):
    """ moves th motor in the endstation"""
    Kappa_Motors.move('th',val,**kwargs)

def dmvth(val):
    """ relative th motor in the endstation"""
    Kappa_Motors.move('th',val,relative=True)

def scanth(start,stop,step,**kwargs):
    """ scans the th motor in the endstation"""
    Kappa_Motors.scan("th",start,stop,step,**kwargs)

def dscanth(start,stop,step):
    """relative scans of the th motor in the endstation """
    Kappa_Motors.scan("th",start,stop,step,relative=True)

### chi ###
def mvchi(val,**kwargs):
    """ moves chi motor in the endstation"""
    Kappa_Motors.move('chi',val,**kwargs)

def dmvchi(val):
    """ relative chi motor in the endstation"""
    Kappa_Motors.move('chi',val,relative=True)

def scanchi(start,stop,step,**kwargs):
    """ scans the chi motor in the endstation"""
    Kappa_Motors.scan("chi",start,stop,step,**kwargs)

def dscanchi(start,stop,step):
    """relative scans of the chi motor in the endstation """
    Kappa_Motors.scan("chi",start,stop,step,relative=True)

### phi ###
def mvphi(val,**kwargs):
    """ moves phi motor in the endstation"""
    Kappa_Motors.move('phi',val,**kwargs)

def dmvphi(val):
    """ relative phi motor in the endstation"""
    Kappa_Motors.move('phi',val,relative=True)

def scanphi(start,stop,step,**kwargs):
    """ scans the phi motor in the endstation"""
    Kappa_Motors.scan("phi",start,stop,step,**kwargs)

def dscanphi(start,stop,step):
    """relative scans of the phi motor in the endstation """
    Kappa_Motors.scan("phi",start,stop,step,relative=True)

### uan moves tth and th simultaneously 
def uan(tth,th):
    """ Moves tth and th motors simultaneously in the in the Kappa chamber
    """
    #move tth and th
    mvth(th,wait=False)
    mvtth(tth,wait=False)
    sleep(0.2)     

    while True:
        status = Kappa_Motors.status()
        if status == 0:
            sleep(0.2)
        else:
            tth_RBV=round(Kappa_Motors.get('tth',verbose=False),3)
            th_RBV=round(Kappa_Motors.get('th',verbose=False),3)
            print('tth='+str(tth_RBV)+' th='+str(th_RBV))
            break

######## mprint and mvsample ###################
def mprint():
    """
    prints current position of the physical motors
    """
    Kappa_Motors.mprint()

def mvsample(position_list):
    """
    moves the sample to the position sepcified by position_list
    position_list = ['description',x,y,z,th,chi,phi]

    Previously: sample
    """
    Kappa_Motors.mvsample(position_list)