from time import sleep
from epics import caget, caput
 


##############################################################################################################
################################            Kappa detector class             ##############################
##############################################################################################################
det_set_pv = '29idKappa:det:set'
det_list = 'd3', 'd4', 'mcp', 'apd', 'yag'

class Kappa_Detector:
    """
    class for Kappa detector
    """
 
    def __init__(self,Kappa_Motors):
        self.get()

    def get(self):
        """
        gets the current det_name and position

        det_name, tth_val
        """
        #det_name = caget('29idKappa:userStringSeq6.STR1')
        det_name = caget(det_set_pv)
        self.name = det_name
        tth_val = Kappa_Motors.get('tth')
        return self.name, tth_val

    def set(self,det_name,move=True):
        """
        sets the tth detector 
        and moves the motor to tth value for the new detecotr
        
        det_names
            d3: large diode
            d4: small diode
            mcp: for the mpa
            apd: avalanche photodiode (not currently installed) 
            yag: yag 'fluorescence screen'
        """
        #get current value for tth
        tth_val = Kappa_Motors.get('tth')

        #change det 
        if det_name in det_list:
            caput(det_set_pv,det_name)
            if move:
                Kappa_Motors.move('tth',tth_val,wait=True,verbose=False)

    def tth0_set(move):
        """
        resetting the tth motor zero to correspond to direct beam

        only works with d4
        """
        current_det=caget(det_set_pv,as_string=True)
        tth_pv = Kappa_Motors._motor_dictionary['th'][3]

        if current_det != 'd4':
            print('tth0 can only be redefined with d4')
        else:
            foo=input('Are you sure you want to reset tth0 (y or n)? >')
            if foo == 'Y' or foo == 'y' or foo == 'yes'or foo == 'YES':
                caput(tth_pv+'.SET','Set')
                sleep(0.5)
                caput(tth_pv+'.VAL',0)
                caput(tth_pv+'.DVAL',0)
                sleep(0.5)
                caput(tth_pv+'.SET','Use')
                print("tth position reset to 0")
            else:
                print("That's ok, everybody can get cold feet in tough situation...")

