
"""
short name for functions used with or without x-rays

"""

import numpy as np
from time import sleep

from epics import PV
from iexcode.instruments.IEX_endstations import *


##############################################################################################################
###########################                      Scan stuff                     ######################
##############################################################################################################
def scan_fillin(VAL,RBV,start,stop,steps_points,**kwargs):
    """
    fills in the scan record for the curretn beamline ioc
    """
    BL.mda.fillin(VAL,RBV,start,stop,steps_points,**kwargs)

def scan_fillin_table(VAL,RBV,my_table,**kwargs):
    """
    fills in the scan record for the curretn beamline ioc
    """
    BL.mda.fillin_table(VAL,RBV,my_table,**kwargs)

def scan_go(**kwargs):
    """
    Starts a scan 
    by default: scanDIM=1  
    Logging is automatic: use **kwargs or the optional logging arguments see scanlog() for details
    kwargs:
        X-ray = True (default), does shutter checks
             = False no shutter checks
    """
    BL.mda.go(verbose=True,**kwargs)


def last_mda():
    """
    returns the last mda file number in the ioc defined by BL_ioc
    Previously: LastMDA
    """
    filenum = BL.mda.lastFileNum()
    return filenum



##############################################################################################################
##############################               Beeper          ##############################
##############################################################################################################


def endstation_cams_enable():
    """
    should live in cameras
    """
    endstation=BL.endstation
    
    cam_dict={'ARPES':[0,1,2],'Kappa':[3,4,6]}  # index of cam_list 
    pvcam1=PV("29id_ps1:cam1:Acquire")
    pvcam2=PV("29id_ps2:cam1:Acquire")
    pvcam3=PV("29id_ps3:cam1:Acquire")
    pvcam4=PV("29id_ps4:cam1:Acquire")
    pvcam6=PV("29id_ps6:cam1:Acquire")
    pvcam7=PV("29id_ps7:cam1:Acquire")
    cam_list=[pvcam1,pvcam2,pvcam3,pvcam4,pvcam6,pvcam7]
    sleep(0.1)
    for pvcam in cam_list:                  # turn OFF all cam
        try:
            if pvcam.connected:  pvcam.put(0)
        except:
            pass
    for i in cam_dict[endstation]:                 # turn ON relevant cam
        try:
            if cam_list[i].connected: cam_list[i].put(1)
            else: print(cam_list[i].pvname+' not connected')
        except:
            pass

