
from epics import caget, caput

from iexcode.instruments.userCalcs import userStringSeq_clear
from iexcode.instruments.AD_utilities import *
from iexcode.instruments.IEX_endstations import *


def cam_pv_dictionary(cam_num):
    """
    dictionary of pv names for the beamline cameras
    """
    d={
        1:"29id_ps1:",
        2:"29id_ps2:",
        3:"29id_ps3:",
        4:"29id_ps4:",
        5:"29idarv5:",
        6:"29id_ps6:",
    }
    return d[cam_num]

def cam_scan_setup(cam_num,ADtype='TIFF',**kwargs):
    """
    sets up the BL scanRecord to trigger the camera (trigger - 2)
    """
    ADplugin = cam_pv_dictionary(cam_num)+ADtype+":"
    ADplugin_ScanSetup(ADplugin,BL.mda, **kwargs)  

def cam_snap(cam_num,ADtype='TIFF',**kwargs):
    """
    takes a camera image and saves it in the user folder
    
    **kwargs:
    ExposureTime: changes both the exposure time and the acquire time for the snapshot
                    resets after acquisition
    FreeRun: True => disable setting and go back to continuous acquision 
                False => leave saving enabled and camera in single acquision

    """
    ADplugin = cam_pv_dictionary(cam_num)+ADtype+":"
    AD_SaveFileSetup(ADplugin,BL.mda,**kwargs)
    AD_snap(ADplugin,**kwargs)

def cam_live(cam_num,ADtype='TIFF',**kwargs):
    """
    puts camera in no save and continuous
    a.k.a free run

    """
    ADplugin = cam_pv_dictionary(cam_num)+ADtype+":"
    AD_FreeRun(ADplugin,**kwargs)
